/*
    Copyright (C) 2005, 2006  Vansteenhuyse Hubert <hvansteenhuyse@yahoo.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "internal.h"
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>

#ifdef __UNIX__
    const char *logfile = "./hxe.log";
#endif
#ifdef __WINDOWS__
    const char *logfile = "H:\\hxe\\bin\\win\\hxe.log";
#endif

void log(char *fmt, ...){
    va_list args;
    FILE *flog = NULL;

    if((flog = fopen(logfile, "a")) != NULL){
        va_start(args, fmt);
        vfprintf(flog, fmt, args);
        fprintf(flog, "\n");
        va_end(args);
        fflush(flog);
        fclose(flog);
    }
}

/*
    converts a hexadecimal value into a decimal
    assumed valid hexadecimal digits
*/
unsigned int htoi( char *value ){
	unsigned int local_index = 0, dec_value = 0;

		while( *(value + local_index ) != 0 ){
			if((*(value + local_index ) >= '0' ) && (*(value + local_index ) <= '9' )){
				dec_value = ( (*(value + local_index ) - '0' )+
				(dec_value * 16 ));
			}else{
				*( value + local_index ) = toupper( *( value + local_index ) );
				dec_value = (((*(value + local_index ) - 'A') + 10 )
				+ ( dec_value * 16 ));
			}

			local_index++;
		}
	return dec_value;
}

/* converts integers into a 2digit hexadecimal string */
void atoh(char ascii, char *outbuf){
    unsigned char ua = (unsigned char)ascii;
    if(ua > 0)
        sprintf(outbuf, "%.2X", ua);
    else
        sprintf(outbuf, "00");
}


/*
  routine to (re)allocate memory. Returns 0 on error (use errno) else 1
  NEEDMEMORY_CHUNK is a fixed amount of bytes defined in internal.h
  !!! value of '*p' is assumed to be 0 on the first routine call,
      results are unpredictable otherwise.
*/
const int NEEDMEMORY_CHUNK = 1024 * 8;
int needmemory(char **p, unsigned long *plen, unsigned long forcedlen){
    char *newp;
    if(!*p){
        *plen = 0;
        newp = (char*)malloc( (forcedlen == 0 ? NEEDMEMORY_CHUNK : forcedlen) );
    }else
        newp = (char*)realloc((char*)*p, (forcedlen == 0 ? *plen + NEEDMEMORY_CHUNK : forcedlen) );

    if(newp){
        if(!forcedlen) *plen += NEEDMEMORY_CHUNK; else *plen = forcedlen;
        *p = (char*)newp;
        return 1;
    }
    return 0;
}

void freememory(void* p){ free(p); }

CStr::CStr(unsigned long bufsze){
    memset(this, 0, sizeof(CStr));
    clear();
    if(bufsze && needmemory(&mBuf, &mSze, bufsze)){
        memset(mBuf, 0, mSze);
    }
}

CStr::~CStr(){ clear(); }

void CStr::clear(){
    if(mBuf) freememory(mBuf);
    memset(this, 0, sizeof(CStr));
}
char* CStr::get(){ return (mBuf ? mBuf : (char*)""); }

unsigned long CStr::getsize(){ return mTail; }

void CStr::del(){
    if(mTail){
        //log("[Size %d], [Tail %d]: '%s'", mSze, mTail, get());
        mTail--;
        mBuf[mTail] = 0;
    }
}

char CStr::get(unsigned long at){ return (mBuf ? (at <= strlen(mBuf) ? mBuf[at] : 0) : 0); }

void CStr::setat(unsigned long at, char ch){
    if(at < mTail)mBuf[at] = ch;
}

int CStr::append(const char *str){
    unsigned long s;
    if(!str) return 0;
    s = strlen(str) + 1;
    if(mTail + s >= mSze && !needmemory(&mBuf, &mSze, mSze + s)) return 0;
    memset(mBuf + mTail, 0, mSze - mTail);
    memcpy(mBuf + mTail, str, s - 1);
    mTail += s - 1;
    //log("[Size %d], [Tail %d]: '%s'", mSze, mTail, get());
    return s - 1;
}

int CStr::append(const char c){
    if(mSze <= mTail && !needmemory(&mBuf, &mSze, mSze + NEEDMEMORY_CHUNK)) return 0;
    memset(mBuf + mTail, 0, mSze - mTail);
    mBuf[mTail] = c;
    mTail++;
    //log("[Size %d], [Tail %d]: '%s'", mSze, mTail, get());
    return 1;
}

int CStr::startsWith(const char*str){
    if(this->mBuf && strncmp(mBuf, str, strlen(str)) == 0) return 1;
    return 0;
}

char *asc[256] = {
    "NUL",      //000
    "SOH",      //001
    "STX",      //002
    "EOX",      //003
    "EOT",      //004
    "ENQ",      //005
    "ACK",      //006
    "BEL",      //007
    "BS ",       //008
    "TAB",      //009
    "LF ",      //010
    "VT ",      //011
    "FF ",      //012
    "CR ",      //013
    "SO ",      //014
    "SI ",      //015
    "DLE",      //016
    "DC1",      //017
    "DC2",      //018
    "DC3",      //019
    "DC4",      //020
    "NAK",      //021
    "SYN",      //022
    "ETB",      //023
    "CAN",      //024
    "EM ",      //025
    "SUB",      //026
    "ESC",      //027
    "FS ",      //028
    "GS ",      //029
    "RS ",      //030
    "US ",      //031
    "SPC",      //032
    "ASC",     //33 == 33 - 127
    "EXT"   //34 == 128 - ...
};
char *AsciiXlat(unsigned char ascii){
    return asc[(ascii <= 32 ? ascii : (ascii < 128 ? 33 : 34))];
}

